"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

exports.default = baseConfig;

var _path = require("path");

var _friendlyErrorsWebpackPlugin = require("friendly-errors-webpack-plugin");

var _friendlyErrorsWebpackPlugin2 = _interopRequireDefault(_friendlyErrorsWebpackPlugin);

var _progressBarWebpackPlugin = require("progress-bar-webpack-plugin");

var _progressBarWebpackPlugin2 = _interopRequireDefault(_progressBarWebpackPlugin);

var _webpack = require("webpack");

var _webpack2 = _interopRequireDefault(_webpack);

var _htmlWebpackPlugin = require("html-webpack-plugin");

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _utils = require("../utils");

var _extractTextWebpackPlugin = require("extract-text-webpack-plugin");

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _loader = require("./loader");

var _uglifyjsWebpackPlugin = require("uglifyjs-webpack-plugin");

var _uglifyjsWebpackPlugin2 = _interopRequireDefault(_uglifyjsWebpackPlugin);

var _util = require("util");

var _forEach = require("lodash/forEach");

var _forEach2 = _interopRequireDefault(_forEach);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CopyWebpackPlugin = require("copy-webpack-plugin"); /*
                                                         * @Author: insane.luojie
                                                         * @Date:   2017-09-18 18:36:03
                                                         * @Last Modified by: insane.luojie
                                                         * @Last Modified time: 2017-12-04 14:57:43
                                                         */

function baseConfig() {
	var envVars = (0, _assign2.default)({
		PRODUCTION: process.env.NODE_ENV == "production",
		"process.env.NODE_ENV": process.env.NODE_ENV
	}, this.options.env.default, this.options.env[process.env.NODE_ENV]);

	var envs = {};
	(0, _forEach2.default)(envVars, function (val, key) {
		envs[key] = (0, _stringify2.default)(val);
	});

	var HTMLPluginConfig = (0, _assign2.default)({
		filename: "index.html",
		template: this.options.appTemplatePath,
		inject: true,
		title: this.options.title,
		chunksSortMode: "dependency"
	}, {
		base: this.options.router.base || "/"
	}, this.options.html);

	var plugins = [new _progressBarWebpackPlugin2.default(), new _htmlWebpackPlugin2.default(HTMLPluginConfig), new _webpack2.default.DefinePlugin(envs),
	// new webpack.NamedChunksPlugin(),
	new _extractTextWebpackPlugin2.default({
		filename: "css/style.[contenthash:8].css" // this.options.build.filenames.css
	})];

	var nodeModulesDir = (0, _path.join)(__dirname, "..", "..", "..", "node_modules");

	var entry = {
		main: (0, _path.resolve)(this.options.buildDir, "app")
	};

	if (this.options.build.dll) {
		plugins.push(new _webpack2.default.DllReferencePlugin({
			context: this.options.rootDir,
			manifest: (0, _path.resolve)(this.options.cacheDir, "vendor-manifest.json")
		}));
	} else {
		entry.vendor = this.vendors();
		var reg = new RegExp(this.vendors().join("|"));
		plugins.push(new _webpack2.default.optimize.CommonsChunkPlugin((0, _defineProperty3.default)({
			name: "vendor",
			minChunks: function minChunks(module, count) {
				return module.context && module.context.indexOf("node_modules") !== -1 && reg.test(module.context);
			}
		}, "minChunks", Infinity)), new _webpack2.default.optimize.CommonsChunkPlugin({
			name: ["runtime"],
			filename: this.options.dev ? "runtime.[hash:8].js" : "runtime.[chunkhash:8].js"
		}), new CopyWebpackPlugin([{
			from: (0, _path.resolve)(this.options.srcDir, this.options.build.contentBase),
			to: (0, _path.resolve)(this.options.buildDir, "dist")
		}]));
	}
	var rules = _loader.createLoaders.call(this);

	var webpackConfig = {
		entry: entry,
		output: {
			path: (0, _path.resolve)(this.options.buildDir, "dist"),
			filename: "[name].[chunkhash:8].js",
			chunkFilename: "[name].[chunkhash:8].js",
			publicPath: (0, _utils.isUrl)(this.options.build.publicPath) ? this.options.build.publicPath : (0, _utils.urlJoin)(this.options.router.base, this.options.build.publicPath)
		},
		plugins: plugins,
		name: "base",
		module: {
			rules: rules
		},
		resolve: {
			modules: ["node_modules", nodeModulesDir],
			extensions: [".js", ".vue", ".json", ".less", ".ts"],
			alias: {
				static: (0, _path.join)(this.options.srcDir, "static"),
				"~": (0, _path.join)(this.options.srcDir),
				"@": (0, _path.resolve)(this.options.buildDir),
				vue$: "vue/dist/vue.esm.js", // 'vue/dist/vue.common.js' for webpack 1,
				lodash: "lodash"
			}
		},
		resolveLoader: {
			modules: ["node_modules", nodeModulesDir]
		},
		devtool: this.options.dev ? "inline-source-map" : "hidden-source-map"
	};

	if (this.options.dev && this.options.devtool) {
		webpackConfig.devtool = this.options.devtool;
	}

	if (this.options.dev) {
		webpackConfig.plugins.push(new _webpack2.default.HotModuleReplacementPlugin(), new _friendlyErrorsWebpackPlugin2.default({
			compilationSuccessInfo: {
				messages: ["up and running here http://127.0.0.1:" + (this.options.port || 8080) + this.options.router.base]
			}
		}));
	} else {
		webpackConfig.plugins.push(new _webpack2.default.HashedModuleIdsPlugin(), new _webpack2.default.optimize.UglifyJsPlugin({
			sourceMap: true,
			output: {
				comments: false
			},
			parallel: true,
			uglifyOptions: {
				ie8: true,
				ecma: 6
			}
		}));
	}

	// 处理dll plugin
	return webpackConfig;
}