"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _path = require("path");

var _fs = require("fs");

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * @Author: insane.luojie
 * @Date:   2017-09-20 11:52:45
 * @Last Modified by: insane.luojie
 * @Last Modified time: 2017-11-28 13:38:44
 */
var pkg = require((0, _path.resolve)("package.json"));

var _default = {
	version: pkg.version,
	mode: "spa",
	host: "127.0.0.1",
	port: 8080,
	buildDir: ".blade",
	bladeDir: (0, _path.resolve)(__dirname, "../app"),
	dev: process.env.NODE_ENV !== "production",
	build: {
		publicPath: "/",
		filename: "",
		chunkFilename: "",
		cssSourceMap: false,
		extractCSS: true,
		dll: false,
		pretty: false,
		postcss: {},
		contentBase: "public",
		// babel loader include
		babelInclude: []
	},
	appTemplatePath: "",
	layout: {
		main: "./layouts/app"
	},
	runtime: {},
	babel: {},
	eslint: {},
	addons: {},
	html: {
		analytic: ""
	},
	css: [],
	plugins: [],
	// global resource
	resources: [],
	tilte: "blade",
	watchers: {
		webpack: {
			ignored: /-dll/
		},
		chokidar: {}
	},
	release: {
		dir: "dist"
	},
	views: {
		notFound: "./layouts/404.vue"
	},
	vendors: [],
	type: "mobile",
	router: {
		mode: "history",
		linkActiveClass: "b-link-active",
		linkExactActiveClass: "b-c-link-active",
		fallback: false,
		base: "/"
	}
};

function loadWebConfig() {
	var path = (0, _path.resolve)(".", "web.config.js");
	if ((0, _fs.existsSync)(path) && require.cache[path]) {
		delete require.cache[path];
	}
	return require(path);
}

exports.default = {
	create: function create(_opts) {
		var _conf = loadWebConfig();

		var opts = (0, _assign2.default)({}, _opts, _conf);

		var overriderNotFound = false;
		if (opts.views && opts.views.notFound) {
			overriderNotFound = true;
		}

		_lodash2.default.defaultsDeep(opts, _default);

		// 如果native打包，强制使用 router.mode = 'hash'; todo

		// 设置根目录
		opts.rootDir = _opts.rootDir ? _opts.rootDir : process.cwd();
		opts.srcDir = _opts.srcDir ? (0, _path.join)(opts.rootDir, _opts.srcDir) : opts.rootDir;
		opts.buildDir = (0, _path.resolve)(opts.rootDir, opts.buildDir);
		opts.cacheDir = (0, _path.join)(opts.buildDir, ".cache");

		if (overriderNotFound) {
			opts.views.notFound = (0, _utils.relativeTo)(opts.buildDir, (0, _path.join)(opts.rootDir, opts.views.notFound));
		}

		var _postcss = {
			sourceMap: opts.build.cssSourceMap,
			plugins: {
				// https://github.com/postcss/postcss-import
				"postcss-import": {
					root: opts.rootDir,
					path: [opts.srcDir, opts.rootDir]
				},
				// https://github.com/postcss/postcss-url
				"postcss-url": {},
				// http://cssnext.io/postcss
				"postcss-cssnext": {}
			}
		};

		if (opts.type == "mobile") {
			_postcss.plugins["postcss-pxtorem"] = {
				rootValue: 20,
				minPixelValue: 2,
				replace: false,
				propList: ["*"]
			};
		}

		// Postcss
		_lodash2.default.defaultsDeep(opts.build.postcss, _postcss);

		opts.build.postcss.plugins = (0, _keys2.default)(opts.build.postcss.plugins).map(function (p) {
			var plugin = require(p);
			return plugin(opts.build.postcss.plugins[p]);
		});

		// 获取 babel, eslint 设置
		opts.babelOptions = _lodash2.default.defaults(opts.babel, {
			babelrc: false,
			cacheDirectory: !!opts.dev
		});
		if (!opts.babelrc && !opts.babel.presets) {
			opts.babelOptions.presets = [require.resolve("babel-preset-vue-app")];
		}

		// If app.html is defined, set the template path to the user template
		opts.appTemplatePath = (0, _path.resolve)(opts.buildDir, "views/app.html");
		if ((0, _fs.existsSync)((0, _path.join)(opts.srcDir, "app.html"))) {
			opts.appTemplatePath = (0, _path.join)(opts.srcDir, "app.html");
		}

		return opts;
	}
};