"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.isMac = exports.isWindows = undefined;

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _create = require("babel-runtime/core-js/object/create");

var _create2 = _interopRequireDefault(_create);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

exports.isUrl = isUrl;
exports.isPureObject = isPureObject;
exports.wp = wp;
exports.r = r;
exports.relativeTo = relativeTo;
exports.createRoutes = createRoutes;
exports.cleanChildrenRoutes = cleanChildrenRoutes;
exports.sequence = sequence;
exports.wChunk = wChunk;
exports.urlJoin = urlJoin;
exports.isObject = isObject;

var _path = require("path");

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * @Author: insane.luojie
 * @Date:   2017-09-20 16:00:05
 * @Last Modified by: insane.luojie
 * @Last Modified time: 2017-11-28 17:26:22
 */

var reqSep = /\//g;
var sysSep = _lodash2.default.escapeRegExp(_path.sep);
var normalize = function normalize(string) {
	return string.replace(reqSep, sysSep);
};
var isWindows = exports.isWindows = /^win/.test(process.platform);
var isMac = exports.isMac = /^darwin/.test(process.platform);

function isUrl(url) {
	return url.indexOf("http") === 0 || url.indexOf("//") === 0;
}

function isPureObject(o) {
	return !Array.isArray(o) && (typeof o === "undefined" ? "undefined" : (0, _typeof3.default)(o)) === "object";
}

/**
 * 路径替换
 * @param  {String} p
 */
function wp() {
	var p = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";

	/* istanbul ignore if */
	if (isWindows) {
		return p.replace(/\\/g, "\\\\");
	}
	return p;
}

/**
 * 解析地址
 * @return {[type]} [description]
 */
function r() {
	var args = Array.prototype.slice.apply(arguments);
	var lastArg = _lodash2.default.last(args);

	if (lastArg.includes("@") || lastArg.includes("~")) {
		return wp(lastArg);
	}

	return wp(_path.resolve.apply(undefined, (0, _toConsumableArray3.default)(args.map(normalize))));
}

function relativeTo() {
	var args = Array.prototype.slice.apply(arguments);
	var dir = args.shift();

	// Resolve path
	var path = r.apply(undefined, (0, _toConsumableArray3.default)(args));

	// Check if path is an alias
	if (path.includes("@") || path.includes("~")) {
		return path;
	}

	// Make correct relative path
	var rp = (0, _path.relative)(dir, path);
	if (rp[0] !== ".") {
		rp = "./" + rp;
	}
	return wp(rp);
}

function split_path(file) {
	return file.replace(/^pages\//, "").replace(/\.vue$/, "").split("/");
}

function createRoutes(files, srcDir) {
	var router = {};

	files.forEach(function (file) {
		// 只解析两层目录
		var filePath = split_path(file);
		var obj = router;
		var name = filePath[0];

		for (var i = 0; i < filePath.length; i++) {
			var key = filePath[i];
			obj = obj[key] || (obj[key] = (0, _create2.default)(null));

			if (key !== "index" && i != 0) {
				name += "-" + key;
			}

			if (!("name" in obj)) {
				obj.name = name;
				obj.path = i === 0 ? "/" + filePath[i] : filePath[i];
				obj.component = "default";
			}
		}
		obj.component = r(srcDir, file);
		obj.chunkName = file.replace(/\.vue$/, "").replace(/(pages\/[^/]+).*/g, "$1");
	});

	var ary = changeDicToAry(router);
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = (0, _getIterator3.default)(ary), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var item = _step.value;

			if (item.name === "index") {
				item.path = "/";
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return ary;
}

function changeDicToAry(router) {
	var result = [];
	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = (0, _getIterator3.default)((0, _keys2.default)(router)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var key = _step2.value;

			var item = router[key];
			if ((typeof item === "undefined" ? "undefined" : (0, _typeof3.default)(item)) === "object" && key !== "children") {
				result.push(item);

				var children = changeDicToAry(item);

				if (children.length > 0) {
					var _iteratorNormalCompletion3 = true;
					var _didIteratorError3 = false;
					var _iteratorError3 = undefined;

					try {
						for (var _iterator3 = (0, _getIterator3.default)(children), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
							var childrenItem = _step3.value;

							if (childrenItem.path === "index") {
								childrenItem.path = "";
								delete item.name;
							}
						}
					} catch (err) {
						_didIteratorError3 = true;
						_iteratorError3 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion3 && _iterator3.return) {
								_iterator3.return();
							}
						} finally {
							if (_didIteratorError3) {
								throw _iteratorError3;
							}
						}
					}

					item.children = children;
				}

				delete router[key];
			}
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	return result;
}

function cleanChildrenRoutes(routes) {
	var isChild = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

	var start = -1;
	var routesIndex = [];
	routes.forEach(function (route) {
		if (/-index$/.test(route.name) || route.name === "index") {
			// Save indexOf 'index' key in name
			var res = route.name.split("-");
			var s = res.indexOf("index");
			start = start === -1 || s < start ? s : start;
			routesIndex.push(res);
		}
	});
	routes.forEach(function (route) {
		route.path = isChild ? route.path.replace("/", "") : route.path;
		if (route.path.indexOf("?") > -1) {
			var names = route.name.split("-");
			var paths = route.path.split("/");
			if (!isChild) {
				paths.shift();
			} // clean first / for parents
			routesIndex.forEach(function (r) {
				var i = r.indexOf("index") - start; //  children names
				if (i < paths.length) {
					for (var a = 0; a <= i; a++) {
						if (a === i) {
							paths[a] = paths[a].replace("?", "");
						}
						if (a < i && names[a] !== r[a]) {
							break;
						}
					}
				}
			});
			route.path = (isChild ? "" : "/") + paths.join("/");
		}
		route.name = route.name.replace(/-index$/, "");
		if (route.children) {
			if (route.children.find(function (child) {
				return child.path === "";
			})) {
				delete route.name;
			}
			route.children = cleanChildrenRoutes(route.children, true);
		}
	});
	return routes;
}

function sequence(tasks, fn) {
	return tasks.reduce(function (promise, task) {
		return promise.then(function () {
			return fn(task);
		});
	}, _promise2.default.resolve());
}

function wChunk() {
	var p = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";

	/* istanbul ignore if */
	if (isWindows) {
		return p.replace(/\//g, "\\\\");
	}
	return p;
}

function urlJoin() {
	return [].slice.call(arguments).join("/").replace(/\/+/g, "/").replace(":/", "://");
}

function isObject(obj) {
	return (typeof obj === "undefined" ? "undefined" : (0, _typeof3.default)(obj)) == "object";
}